#!/usr/bin/python
# *************************************************************************************************
#
# Horde3D
#   Next-Generation Graphics Engine
#
# Sample Application
# --------------------------------------
# Copyright (C) 2006-2008 Nicolas Schulz
#               2008 Florian Noeding
#
#
# This sample source file is not covered by the LGPL as the rest of the SDK
# and may be used without any restrictions
#
# *************************************************************************************************

import pyglet
import pyglet.gl
import pyglet.window
import pyglet.clock

import horde3d as h3d

from math import sin, cos, radians, degrees
from app import App



class RTSCamera(object):
	''' RTS style strategic zoom
	
		uses a pitch node and a camera node attached to this pitch node to simplify control
	'''
	def __init__(self, renderPipe, initialPosition, initialPitch=-90, initialYaw=0, name='Camera'):
		self._positionNode = h3d.addGroupNode(h3d.RootNode, '%s-position' % name)
		self._cameraNode = h3d.addCameraNode(self._positionNode, '%s-pitch' % name, renderPipe)

		self.wantPosition = self.position = initialPosition
		self.wantPitch = self.pitch = initialPitch
		self.wantYaw = self.yaw = initialYaw


	def zoomIn(self):
		self.wantPosition[1] -= 100

	def zoomOut(self):
		self.wantPosition[1] += 100


	def moveLeft(self):
		# FIXME depends on yaw
		self.wantPosition[0] -= 100

	def moveRight(self):
		# FIXME depends on yaw
		self.wantPosition[0] += 100


	def moveForward(self):
		# FIXME depends on yaw
		self.wantPosition[2] -= 100

	def moveBackward(self):
		# FIXME depends on yaw
		self.wantPosition[2] += 100


	def pitchDown(self):
		self.wantPitch -= 3.5

	def pitchUp(self):
		self.wantPitch += 3.5


	def yawLeft(self):
		self.wantYaw += 5

	def yawRight(self):
		self.wantYaw -= 5


	def _interpolatePosition(self, dt):
		cx, cy, cz = self.position
		wx, wy, wz = self.wantPosition

		dx = wx - cx
		dy = wy - cy
		dz = wz - cz

		# very simple for now: linear interpolation
		nx = cx + dx * dt / 0.125
		ny = cy + dy * dt / 0.125
		nz = cz + dz * dt / 0.125


		return (nx, ny, nz)


	def _interpolateAngles(self, dt):
		cp, cy = self.pitch, self.yaw
		wp, wy = self.wantPitch, self.wantYaw

		dp = wp - cp
		dy = wy - cy

		# very simple for now: linear interpolation
		np = cp + dp * dt / 0.125
		ny = cy + dy * dt / 0.125

		return (np, ny)


	
	def updateNodes(self, dt):
		# calculate current values
		x, y, z = self._interpolatePosition(dt)
		pitch, yaw = self._interpolateAngles(dt)

		# transform
		h3d.setNodeTransform(self._positionNode,
				x, y, z,
				0, 0, 0,
				1, 1, 1)

		h3d.setNodeTransform(self._cameraNode,
				0, 0, 0,
				pitch, yaw, 0,
				1, 1, 1)

		# write values back
		self.position = [x, y, z]
		self.pitch = pitch
		self.yaw = yaw


