#!/usr/bin/python
# *************************************************************************************************
#
# Horde3D
#   Next-Generation Graphics Engine
#
# Sample Application
# --------------------------------------
# Copyright (C) 2006-2008 Nicolas Schulz
#               2008 Florian Noeding
#
#
# This sample source file is not covered by the LGPL as the rest of the SDK
# and may be used without any restrictions
#
# *************************************************************************************************

import pyglet
import pyglet.gl
import pyglet.window
import pyglet.clock

import horde3d as h3d

from math import sin, cos, radians, degrees
from app import App
from rtscamera import RTSCamera



class TerrainApp(App):
	def __init__(self):
		super(TerrainApp, self).__init__()

		self.camPos = [512, 120, 512]
		self.camRot = [0, -45, 0]
		self.camVel = 0

	def _onResize(self, width, height):
		super(TerrainApp, self)._onResize(width, height)

		h3d.setupCameraView(self._camera, 45, width / float(height), 0.1, 30000)



	def _h3dAddResources(self):
		super(TerrainApp, self)._h3dAddResources()

		self._h3dres.terrain = h3d.addResource(h3d.ResourceTypes.SceneGraph, 'terrain.scene.xml', 0)

	def _h3dSetupCamera(self):
		# add camera
		self._rtsCamera = RTSCamera(self._h3dres.forwardPipe, [512, 256, 512])
		self._camera = self._rtsCamera._cameraNode

	
	def _h3dSetupScene(self):
		# add terrain
		self._terrain = h3d.addNodes(h3d.RootNode, self._h3dres.terrain)

		# set sun direction for ambient pass of terrain
		matRes = h3d.findResource(h3d.ResourceTypes.Material, 'terrain/terrain.material.xml')
		h3d.setMaterialUniform(matRes, 'sunDir', 1, -1, 0, 0)

	
	def _onMouseScroll(self, x, y, scrollX, scrollY):
		rtsCam = self._rtsCamera
		
		if scrollY == 0:
			return
		elif scrollY < 0:
			rtsCam.zoomOut()
		else:
			rtsCam.zoomIn()




	def _mainloopUpdate(self, dt):
		key = pyglet.window.key

		if self.keyboard[key.LCTRL] or self.keyboard[key.RCTRL]:
			if self.keyboard[key.UP]:
				self._rtsCamera.pitchDown()
			if self.keyboard[key.DOWN]:
				self._rtsCamera.pitchUp()
			if self.keyboard[key.LEFT]:
				self._rtsCamera.yawLeft()
			if self.keyboard[key.RIGHT]:
				self._rtsCamera.yawRight()
		else:
			if self.keyboard[key.LEFT]:
				self._rtsCamera.moveLeft()
			if self.keyboard[key.RIGHT]:
				self._rtsCamera.moveRight()
			if self.keyboard[key.UP]:
				self._rtsCamera.moveForward()
			if self.keyboard[key.DOWN]:
				self._rtsCamera.moveBackward()


		self._rtsCamera.updateNodes(dt)

		pass
		

def main():
	app = TerrainApp()
	app.init()
	app.mainloop()


if __name__ == '__main__':
	main()


